package br.com.centralit.api.dao.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.EstruturaOrganizacionalDao;
import br.com.centralit.api.model.EstruturaOrganizacional;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.model.arquitetura.PersistentObject;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Search;
import com.googlecode.genericdao.search.Sort;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: EstruturaOrganizacionalDaoHibernate</b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 10/12/2014 - 15:44:22
 *
 * @version 1.0.0
 *
 * @author renato.jesus
 *
 */
@Repository("estruturaOrganizacionalDao")
@SuppressWarnings("unchecked")
public class EstruturaOrganizacionalDaoHibernate extends CitGenericDAOImpl implements EstruturaOrganizacionalDao {

	public EstruturaOrganizacionalDaoHibernate() {

		super(EstruturaOrganizacional.class);
	}

	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionais(String nome) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"), Filter.ilike("codigo", "%" + nome + "%"));

		search.addSort(Sort.asc("nome"));

		search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		search.setMaxResults(10);

		return this.search(search, this.persistentClass);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por buscar todas as estruturas organizacionais administrativas
	 *
	 * @author luis.camargo
	 *
	 * @param nome
	 * @param idOrganizacao
	 * @return
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisAdministrativas(String nome, Long idOrganizacao, boolean trazerBloqueados) {

		SearchSeven search = new SearchSeven(this.persistentClass, trazerBloqueados);

		search.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"), Filter.ilike("codigo", "%" + nome + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEqual("dominioTipoEstruturaOrganizacional.codigo", 2);

		search.addSort(Sort.asc("nome"));

		if(!trazerBloqueados)
			search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		search.setMaxResults(10);

		return this.search(search, this.persistentClass);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por buscar todas as estruturas organizacionais nivel 0 (zero)
	 *
	 * @author renato.jesus
	 *
	 * @param idOrganizacao
	 * @return
	 */
	@Override
	public List<EstruturaOrganizacional> findParents(Long idOrganizacao) {

		// Não foi utilizado SearchSeven por necessitar listar estruturas com dataBloqueio
		Search search = new Search(this.persistentClass);

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEmpty("estruturaOrganizacionalParent.id");

		search.addFilterEmpty("dataInativo");

		search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		return this.search(search, this.persistentClass);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por buscar todas as estruturas organizacionais filhas com filtro datafim ou não
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param idOrganizacao
	 * @param exibirEstruturasAtivas
	 * @return List<EstruturaOrganizacional>
	 */
	@Override
	public List<EstruturaOrganizacional> findChildrens(Long idOrganizacao, Boolean exibirEstruturasAtivas) {

		// Não foi utilizado SearchSeven por necessitar listar estruturas com dataBloqueio
		Search search = new Search(this.persistentClass);

		if (exibirEstruturasAtivas) {

			search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));
		}

		search.addFilterEqual("estruturaOrganizacionalParent.id", idOrganizacao);

		search.addSortAsc("nome");

		return this.search(search, this.persistentClass);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por validar se uma estrutura possui filhos
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param idOrganizacao
	 * @return boolean
	 */
	@Override
	public Boolean possuiFilhos(Long idOrganizacao) {

		// Não foi utilizado SearchSeven por necessitar listar estruturas com dataBloqueio
		Search search = new Search(this.persistentClass);

		search.addFilterEqual("estruturaOrganizacionalParent.id", idOrganizacao);

		return this.count(search) > 0;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisPorOrganizacao(String nome, Long idOrganizacao) {

		SearchSeven search = new SearchSeven(this.persistentClass);

//		search.addFilterILike("nome", "%" + nome + "%");

		search.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"), Filter.ilike("codigo", "%" + nome + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addSort(Sort.asc("nome"));

		search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		search.setMaxResults(10);

		return this.search(search, this.persistentClass);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisFilhasPorOrganizacao(String nome, long idOrganizacao) {

		Query query = null;
		switch (UtilDataBase.getDataBaseName(em())){
			case POSTGRESQL :
				StringBuilder queryPostgres = new StringBuilder();

				queryPostgres.append("WITH RECURSIVE rel_tree AS ");
				queryPostgres.append("(SELECT estruturaorganizacional.id, organizacao.nome, organizacao.isOrgao, codigo, classificacao, dataReferenciaVigente, configuracao_id, dataBloqueio, dataFim, dataInicio, possuiBemPatrimonial, sigla, unidadesuperior_id, localizacao_id, tipoestruturaorganizacional_id,estruturaOrganizacionalResponsavel_id, organizacao_id, diasRequisicao, unidadeConsumidoraRequisitante, almoxarifado, datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, 1 AS level, array[estruturaorganizacional.id] AS path_info ");
				queryPostgres.append("FROM estruturaorganizacional ");
				queryPostgres.append("INNER Join  organizacao on organizacao.id = estruturaorganizacional.id ");
				queryPostgres.append("WHERE organizacao_id = " + idOrganizacao + " AND (dataFim is null OR dataFim >= TO_DATE('"+ UtilDate.getDataAtualSemHoras() +"', 'yyyy-MM-dd') ) AND dataBloqueio IS NULL AND (lower(nome) LIKE lower('%" + nome + "%') OR lower(sigla) LIKE lower('%" + nome + "%') OR lower(classificacao) LIKE lower('%" + nome + "%') ) ");
				queryPostgres.append("UNION ALL ");
				queryPostgres.append("SELECT e.id, organizacao.nome, organizacao.isOrgao, e.codigo, e.classificacao, organizacao.dataReferenciaVigente, e.configuracao_id, organizacao.dataBloqueio, e.dataFim, e.dataInicio, e.possuiBemPatrimonial, organizacao.sigla, e.unidadesuperior_id, e.localizacao_id, e.tipoestruturaorganizacional_id, e.estruturaOrganizacionalResponsavel_id, e.organizacao_id, e.diasRequisicao, e.unidadeConsumidoraRequisitante, e.almoxarifado, organizacao.datacriacao, organizacao.dataedicao, organizacao.datainativo, organizacao.autor_id, organizacao.editor_id, organizacao.version, organizacao.inativador_id, p.level + 1, p.path_info||e.id ");
				queryPostgres.append("FROM estruturaorganizacional e ");
				queryPostgres.append("INNER JOIN  organizacao  on organizacao.id = e.id ");
				queryPostgres.append("JOIN rel_tree p ON p.id = e.organizacao_id ) ");
				queryPostgres.append("SELECT DISTINCT ON (id) id, nome, isOrgao, codigo, classificacao, dataReferenciaVigente, configuracao_id, dataBloqueio, dataFim, dataInicio, possuiBemPatrimonial, sigla, unidadesuperior_id, localizacao_id, tipoestruturaorganizacional_id, estruturaOrganizacionalResponsavel_id, organizacao_id, diasRequisicao, unidadeConsumidoraRequisitante, almoxarifado, datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, level ");
				queryPostgres.append("FROM rel_tree WHERE dataBloqueio IS NULL AND dataInativo IS NULL ORDER BY id, level DESC");
				query = em().createNativeQuery(queryPostgres.toString(), EstruturaOrganizacional.class);
		return query.getResultList();
			case SQL_SERVER :
				StringBuilder querySqlServer = new StringBuilder();

				querySqlServer.append("WITH estruturaorganizacional_CTE AS ")
							.append("SELECT estruturaorganizacional.id, organizacao.nome, organizacao.isOrgao, codigo, classificacao, organizacao.dataReferenciaVigente, ")
							.append("configuracao_id, organizacao.dataBloqueio, dataFim, dataInicio, possuiBemPatrimonial, sigla, unidadesuperior_id, localizacao_id, ")
							.append("tipoestruturaorganizacional_id, estruturaOrganizacionalResponsavel_id, organizacao_id, diasRequisicao, ")
							.append("unidadeConsumidoraRequisitante, almoxarifado, organizacao.datacriacao, organizacao.dataedicao, organizacao.datainativo, organizacao.autor_id, "
									+ "organizacao.editor_id, ")
							.append("version, organizacao.inativador_id, 1 AS level")
							.append("FROM estruturaOrganizacional INNER Join organizacao ON organizacao.id = estruturaorganizacional.id ")
							.append("WHERE organizacao_id =" + idOrganizacao + "AND (dataFim is null OR dataFim >= CAST('"+ UtilDate.getDataAtualSemHoras() +"' AS date) AND dataBloqueio IS NULL AND ")
							.append("(lower(nome) LIKE lower('%" + nome + "%') OR lower(sigla) LIKE lower('%" + nome + "%') OR lower(classificacao) LIKE lower('%" + nome + "%'))) ")
							.append("UNION ALL ")
							.append("SELECT e.id, organizacao.nome, organizacao.isOrgao, e.codigo, e.classificacao, organizacao.dataReferenciaVigente, e.configuracao_id, ")
							.append("organizacao.dataBloqueio, e.dataFim, e.dataInicio, e.possuiBemPatrimonial, organizacao.sigla, e.unidadesuperior_id, ")
							.append("e.localizacao_id, e.tipoestruturaorganizacional_id, e.estruturaOrganizacionalResponsavel_id, e.organizacao_id, e.diasRequisicao, ")
							.append("e.unidadeConsumidoraRequisitante, e.almoxarifado, organizacao.datacriacao, organizacao.dataedicao, organizacao.datainativo, ")
							.append("organizacao.autor_id, organizacao.editor_id, organizacao.version, organizacao.inativador_id, p.level + 1 ")
							.append("FROM estruturaorganizacional e INNER JOIN organizacao ON organizacao.id = e.id ")
							.append("JOIN estruturaorganizacional_CTE p ON p.id = e.organizacao_id ")
							.append(")")
							.append("SELECT DISTINCT id, nome, isOrgao, codigo, classificacao, dataReferenciaVigente, configuracao_id, dataBloqueio, dataFim, dataInicio, ")
							.append("possuiBemPatrimonial, sigla, unidadesuperior_id, localizacao_id, tipoestruturaorganizacional_id, ")
							.append("estruturaOrganizacionalResponsavel_id, organizacao_id, diasRequisicao, unidadeConsumidoraRequisitante, almoxarifado, ")
							.append("datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, level ")
							.append("FROM estruturaorganizacional_CTE WHERE dataBloqueio IS NULL AND dataInativo IS NULL ORDER BY id, level DESC ");
				query = em().createNativeQuery(querySqlServer.toString(), EstruturaOrganizacional.class);
				return query.getResultList();
			case ORACLE :
			default:
				throw new IllegalArgumentException("N&atilde;o implementado para o banco de dados da conex&atilde;o atual. Verifique a "
							+ "conex&atilde;o e tente novamente.");
	}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar todas as <code>EstruturaOrganizacional</code> de uma <code>Organizacao</code>
	 *
	 * @author geovane.filho
	 *
	 * @param idOrganizacao
	 *            ID da Organizacao a se obter as Estruturas Organizacionais.
	 *
	 * @return <code>List<EstruturaOrganizacional></code>
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisPorOrganizacao(Long idOrganizacao) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addSort(Sort.asc("nome"));

		search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		return this.search(search, this.persistentClass);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por verificar se a localização possui uma estrutura vinculada
	 *
	 * @author iago.almeida
	 *
	 * @param localizacaoId
	 * @return
	 */
	@Override
	public boolean existeEstruturaVinculadaALocacalizacao(Long localizacaoId, Usuario usuarioLogado) {

		SearchSeven search = new SearchSeven();

		search.addFilterOr(Filter.equal("organizacao.id", usuarioLogado.getOrganizacao().getId()), Filter.equal("id", usuarioLogado.getOrganizacao().getId()));

		search.addFilterIn("localizacao.id", localizacaoId);

		return this.count(search) > 0 ? Boolean.TRUE : Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar estruturas organizacionais na tree por nome
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param nome
	 * @return List<EstruturaOrganizacional>
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisPorNomeTree(String nome) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterILike("nome", "%" + nome + "%");

		search.addSort(Sort.asc("nome"));

		return this.search(search, this.persistentClass);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar estruturas organizacionais da tree pelo id da órgão do usuário logado e nome
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param idEstrutura
	 * @param nome
	 * @param exibirEstruturasAtivas
	 * @return List<EstruturaOrganizacional>
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisDaTree(Long idOrganizacao, String nome, Boolean exibirEstruturasAtivas) {

		StringBuilder strQuery = new StringBuilder();
		Query query;

		switch (UtilDataBase.getDataBaseName(em())){

			case POSTGRESQL :
				strQuery.append("WITH RECURSIVE rel_tree AS ");
				strQuery.append("(SELECT id, nome, codigo, classificacao, "
						+ "dataBloqueio, dataFim, dataInicio, sigla,"
						+ " unidadesuperior_id,  unidadegestora_id,  localizacao_id, tipoestruturaorganizacional_id, organizacao_id,"
						+ " datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, 1 AS level, array[id] AS path_info ");
				strQuery.append("FROM estruturaorganizacional ");

				strQuery.append("WHERE organizacao_id = " + idOrganizacao);

				if(exibirEstruturasAtivas) {
					strQuery.append(" AND ((dataFim is null OR dataFim >= TO_DATE('"+ UtilDate.getDataAtualSemHoras() +"', 'yyyy-MM-dd')) AND (dataBloqueio IS NULL OR dataBloqueio >= TO_DATE('"+ UtilDate.getDataAtualSemHoras() +"', 'yyyy-MM-dd')))");
				}

				strQuery.append(" AND (lower(nome) LIKE lower('%" + nome + "%') OR lower(sigla) LIKE lower('%" + nome + "%') OR lower(codigo) LIKE lower('%" + nome + "%') ) ");
				strQuery.append("AND dataInativo IS NULL ");

				strQuery.append("UNION ALL ");
				strQuery.append("SELECT e.id, e.nome, e.codigo, e.classificacao, "
						+ "e.dataBloqueio, e.dataFim, e.dataInicio, e.sigla, "
						+ "e.unidadesuperior_id, e.unidadegestora_id, e.localizacao_id, e.tipoestruturaorganizacional_id, e.organizacao_id, "
						+ "e.datacriacao, e.dataedicao, e.datainativo, e.autor_id, e.editor_id, e.version,"
						+ "e.inativador_id, p.level + 1, p.path_info||e.id ");
				strQuery.append("FROM estruturaorganizacional e ");
				strQuery.append("JOIN rel_tree p ON e.id = p.unidadesuperior_id ) ");
				strQuery.append("SELECT DISTINCT ON (id) id, nome, codigo, classificacao, "
						+ "dataBloqueio, dataFim, dataInicio, sigla, unidadesuperior_id, unidadegestora_id, localizacao_id, tipoestruturaorganizacional_id, "
						+ "organizacao_id, datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, level ");
				strQuery.append("FROM rel_tree WHERE (dataBloqueio IS NULL OR dataBloqueio >= TO_DATE('"+ UtilDate.getDataAtualSemHoras() +"', 'yyyy-MM-dd')) AND dataInativo IS NULL ORDER BY id, level DESC");

				query = em().createNativeQuery(strQuery.toString(), EstruturaOrganizacional.class);
			return query.getResultList();

			case SQL_SERVER :
				strQuery.append("WITH rel_tree (id, nome, codigo, classificacao, dataBloqueio, ")
				.append("dataFim, dataInicio, sigla, unidadesuperior_id, unidadegestora_id, localizacao_id, tipoestruturaorganizacional_id, ")
				.append("organizacao_id, datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, level )")
				.append("as (SELECT id, nome, codigo, classificacao, ")
				.append("dataBloqueio, dataFim, dataInicio, sigla, ")
				.append("unidadesuperior_id, localizacao_id, tipoestruturaorganizacional_id, organizacao_id, ")
				.append("datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id , 1 AS level FROM estruturaorganizacional estruturaorganizacional ");

				strQuery.append("WHERE organizacao_id = " + idOrganizacao);
				if(exibirEstruturasAtivas) {
					strQuery.append(" AND ((dataFim is null OR dataFim >= CAST('"+ UtilDate.getDataAtualSemHoras() +"' AS date)) AND (dataBloqueio IS NULL OR dataBloqueio >= TO_DATE('"+ UtilDate.getDataAtualSemHoras() +"', 'yyyy-MM-dd')))");
				}
				strQuery.append(" AND (lower(nome) LIKE lower('%" + nome + "%') OR lower(sigla) LIKE lower('%" + nome + "%')) ");
				strQuery.append(" AND dataInativo IS NULL ");

				strQuery.append("UNION ALL ");
				strQuery.append("SELECT e.id, e.nome, e.codigo, e.classificacao, ")
				.append("e.dataBloqueio, e.dataFim, e.dataInicio, e.sigla, ")
				.append("e.unidadesuperior_id, e.unidadegestora_id,  e.localizacao_id, e.tipoestruturaorganizacional_id, e.organizacao_id, ")
				.append("e.datacriacao, e.dataedicao, e.datainativo, e.autor_id, e.editor_id, e.version, ")
				.append("e.inativador_id , level + 1 FROM estruturaorganizacional e ")
				.append("JOIN rel_tree p ON e.id = p.unidadesuperior_id ) ");
				strQuery.append("SELECT DISTINCT (id) id, nome, codigo, classificacao, ")
				.append("dataBloqueio, dataFim, dataInicio, sigla, unidadesuperior_id, unidadegestora_id, localizacao_id, tipoestruturaorganizacional_id, ")
				.append("organizacao_id, datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id ")
				.append("FROM rel_tree WHERE (dataBloqueio IS NULL OR dataBloqueio >= TO_DATE('"+ UtilDate.getDataAtualSemHoras() +"', 'yyyy-MM-dd')) AND dataInativo IS NULL ORDER BY id DESC ");

				query = em().createNativeQuery(strQuery.toString(), EstruturaOrganizacional.class);

				return query.getResultList();

			//TODO Implementar sql nativa para os outros bancos
			case ORACLE :
			case MYSQL :
			case DB2 :
			default:
				throw new IllegalArgumentException("SQL inexistente para o SGBD");
			}

	}

	@Override
	public PersistentObject saveIfNotExist(PersistentObject entity) {

		Organizacao organizacao = (Organizacao) entity;

		Search search = new Search();

		search.addFilterEqual("nome", organizacao.getNome());

		search.addFilterEqual("sigla", organizacao.getSigla());

		search.setMaxResults(1);

		PersistentObject obj = searchUnique(search);

		if (UtilObjeto.isReferencia(obj)) {

			return obj;

		} else {

			return this.save(entity);

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar estruturas organizacionais por nome das estruturas inventariaveis
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param nome
	 * @param idEstruturas
	 * @return List<EstruturaOrganizacional>
	 */
	@Override
	public List<EstruturaOrganizacional> findEstruturaOrganizacionalPorNomeInventario(String nome, List<Long> idEstruturas) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterIn("id", idEstruturas);

		searchSeven.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"));

		searchSeven.addSort(Sort.asc("nome"));

		searchSeven.setMaxResults(10);

		return this.search(searchSeven);

	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por listar estruturas localizadoras por inventário
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param busca
	 * @param idInventario
	 * @return List<EstruturaOrganizacional>
	 */
	public List<EstruturaOrganizacional> findULInventario(String busca, Long idInventario) {

		StringBuilder query = new StringBuilder();
		query.append("SELECT * ")
		.append(" FROM ESTRUTURAORGANIZACIONAL eo")
		.append(" WHERE eo.id in (")
		.append(" SELECT eo.id from inventestruturaorganizacional ieo")
		.append(" INNER JOIN estruturaorganizacional eo on eo.id = ieo.estruturaorganizacional_id")
		.append(" WHERE ieo.inventario_id = :idInventario")
		.append(")")
		.append(" AND (lower(nome) LIKE lower('%" + busca + "%') OR lower(sigla) LIKE lower('%" + busca + "%') OR lower(classificacao) LIKE lower('%" + busca + "%') OR lower(codigo) LIKE lower('%" + busca + "%'))" );

		Query resultado = em().createNativeQuery(query.toString(), EstruturaOrganizacional.class);
		resultado.setParameter("idInventario", idInventario);
		resultado.setMaxResults(10);

		return resultado.getResultList();
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por listar estruturas administrativas por inventário
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param busca
	 * @param idInventario
	 * @return List<EstruturaOrganizacional>
	 */
	@Override
	public List<EstruturaOrganizacional> findUAInventario(String busca, Long idInventario) {

		StringBuilder query = new StringBuilder();
		query.append("SELECT * ")
		.append(" FROM ESTRUTURAORGANIZACIONAL eo")
		.append(" WHERE eo.id in (")
		.append(" SELECT parent.id from inventestruturaorganizacional ieo")
		.append(" INNER JOIN estruturaorganizacional estrutura on estrutura.id = ieo.estruturaorganizacional_id")
		.append(" INNER JOIN estruturaorganizacional parent ON parent.id = estrutura.unidadesuperior_id")
		.append(" WHERE ieo.inventario_id = :idInventario")
		.append(")")
		.append(" AND (lower(eo.nome) LIKE lower('%" + busca + "%') OR lower(eo.sigla) LIKE lower('%" + busca + "%') OR lower(eo.classificacao) LIKE lower('%" + busca + "%') OR lower(eo.codigo) LIKE lower('%" + busca + "%'))" );

		Query resultado = em().createNativeQuery(query.toString(), EstruturaOrganizacional.class);
		resultado.setParameter("idInventario", idInventario);
		resultado.setMaxResults(10);

		return resultado.getResultList();
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por buscar uma estrutura organizacional por entrada patrimonio
	 *
	 * @author luis.camargo
	 *
	 * @param idEntrada
	 * @return EstruturaOrganizacional
	 */
	@Override
	public EstruturaOrganizacional findByEntradaPatrimonio(Long idEntrada) {

		StringBuilder query = new StringBuilder();
		query.append("SELECT EO.* ")
		.append(" FROM ESTRUTURAORGANIZACIONAL EO ")
		.append(" INNER JOIN ENTRADAPATRIMONIO EP ON EP.ESTRUTURAORGANIZACIONAL_ID = EO.ID ")
		.append(" WHERE EP.ID = :idEntrada ");
		Query resultado = em().createNativeQuery(query.toString(), EstruturaOrganizacional.class);
		resultado.setParameter("idEntrada", idEntrada);

		return (EstruturaOrganizacional) resultado.getSingleResult();
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por
	 *
	 * @author vinicius.deoliveira
	 *
	 * @param idsEstruturas
	 * @return
	 */
	@Override
	public List<EstruturaOrganizacional> findEstruturaOrganizacionalParent(List<Long> idEstruturas){
		SearchSeven searchSeven = new SearchSeven();
		searchSeven.addFilterIn("estruturaOrganizacionalParent.id", idEstruturas);
		return this.search(searchSeven);
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por
	 *
	 * @author vinicius.deoliveira
	 *
	 * @param idInventario
	 * @return
	 */
	@Override
	public EstruturaOrganizacional findEstruturaInventario(Long idInventario){
		StringBuilder query = new StringBuilder();
		query.append("SELECT * ")
		.append(" FROM ESTRUTURAORGANIZACIONAL eo")
		.append(" INNER JOIN inventestruturaorganizacional inventarioestrutura on inventarioestrutura.estruturaorganizacional_id = eo.id")
		.append(" WHERE inventarioestrutura.inventario_id = :idInventario");

		Query resultado = em().createNativeQuery(query.toString(), EstruturaOrganizacional.class);
		resultado.setParameter("idInventario", idInventario);
		resultado.setMaxResults(1);

		return (EstruturaOrganizacional) resultado.getSingleResult();
	}

	/**
	 * Método responsável por listar <code>EstruturaOrganizacional</code> de uma <code>Organizacao</code> com isAlmoxarifado
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param idOrganizacao
	 * @return <code>List<EstruturaOrganizacional></code>
	 */
	@Override
	public List<Long> listarIdEstruturasOrganizacionaisAlmoxarifadoPorOrganizacao(Long idOrganizacao) {

		StringBuilder strQuery = new StringBuilder();
		strQuery.append("select eoexterno.id from estruturaorganizacional eoexterno ")
		.append(" where eoexterno.id in (")
		.append(" select eo.id from EstruturaOrganizacionalPatrim al")
		.append(" INNER JOIN ESTRUTURAORGANIZACIONAL eo ON eo.id = al.estruturaorganizacional_id")
		.append(" WHERE al.isAlmoxarifado IS TRUE")
		.append(" AND (eo.dataFim IS NULL OR eo.dataFim > :dataHoje)")
		.append(" AND eo.dataInativo IS NULL")
		.append(" AND al.dataBloqueio IS NULL")
		.append(" AND eo.organizacao_id = :organizacaoId")
		.append(" )");

		Query query = this.em().createNativeQuery(strQuery.toString());
		query.setParameter("dataHoje", Calendar.getInstance());
		query.setParameter("organizacaoId", idOrganizacao);

		return query.getResultList();
	}

	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisLocalizadora(String nome, Long idOrganizacao, boolean trazerBloqueados) {

		SearchSeven search = new SearchSeven(this.persistentClass, trazerBloqueados);

		search.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"), Filter.ilike("codigo", "%" + nome + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEqual("dominioTipoEstruturaOrganizacional.codigo", 3);

		search.addSort(Sort.asc("nome"));

		if(!trazerBloqueados)
			search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		search.setMaxResults(10);

		return this.search(search, this.persistentClass);
	}

	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisGestora(String nome, Long idOrganizacao, boolean trazerBloqueados) {

		SearchSeven search = new SearchSeven(this.persistentClass, trazerBloqueados);

		search.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"), Filter.ilike("codigo", "%" + nome + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEqual("dominioTipoEstruturaOrganizacional.codigo", 1);

		search.addSort(Sort.asc("nome"));

		if(!trazerBloqueados)
			search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		search.setMaxResults(10);

		return this.search(search, this.persistentClass);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar <code>EstruturaOrganizacional</code> de uma <code>Organizacao</code> a partir do nome da <code>EstruturaOrganizacional</code> e com filtro not in de ids
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param nome
	 * @param idOrganizacao
	 * @param idsEstruturasNotIn
	 * @return <code>List<EstruturaOrganizacional></code>
	 */
	@Override
	public List<EstruturaOrganizacional> listarEstruturasOrganizacionaisPorOrganizacaoNotIn(String nome, Long idOrganizacao, List<Long> idsEstruturasNotIn) {

		SearchSeven search = new SearchSeven();

		if(!UtilColecao.isVazio(idsEstruturasNotIn)){
			search.addFilterNotIn("id", idsEstruturasNotIn);
		}

		search.addFilterOr(Filter.ilike("nome", "%" + nome + "%"), Filter.ilike("classificacao", "%" + nome + "%"), Filter.ilike("codigo", "%" + nome + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addSort(Sort.asc("nome"));

		search.addFilterOr(Filter.greaterOrEqual("dataFim", UtilDate.getCalendarDaData(UtilDate.getDataAnterior(new Date()))), Filter.isEmpty("dataFim"));

		search.setMaxResults(10);

		return this.search(search);
	}
	
}