package br.com.centralit.api.dao.impl;

import java.util.HashMap;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.PessoaDao;
import br.com.centralit.api.model.Pessoa;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Filter;
import br.com.centralit.framework.model.SearchParams;
import br.com.centralit.framework.util.UtilString;

import com.googlecode.genericdao.search.Search;
import com.googlecode.genericdao.search.SearchResult;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>PessoaDaoHibernate
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * @since 28/11/2014 - 11:41:49
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Repository("pessoaDao")
public class PessoaDaoHibernate extends CitGenericDAOImpl implements PessoaDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param classs
	 */
	public PessoaDaoHibernate() {

		super(Pessoa.class);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Pessoa> findByUsuario(Pessoa pessoa) {
		String queryString = "from Pessoa pessoa where usuario.id = :idUsuario ";
		if (!pessoa.isNew()) {
			queryString += "and pessoa.id <> :idPessoa ";
		}
		Query query = em().createQuery(queryString);
		query.setParameter("idUsuario", pessoa.getUsuario().getId());
		if (!pessoa.isNew()) {
			query.setParameter("idPessoa", pessoa.getId());
		}

		return query.getResultList();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Pessoa buscaPessoaPorUsuarioId(Long idUsuario) {
		String queryString = "from Pessoa pessoa  where usuario.id = :idUsuario ";
		Query query = em().createQuery(queryString);
		query.setParameter("idUsuario", idUsuario);
		 List lista = query.getResultList();
		 if(!lista.isEmpty()){
			 return (Pessoa) lista.get(0);
		 }else{
			 return null;
		 }
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public SearchResult findAllPessoas(SearchParams params){
		
		Filter filter = params.getFilters().remove(2);
		
		if(params.getKeywordValue() != null && !params.getKeywordValue().equals("")){
			filter.setValue(null);
			
			params.getFilters().add(new Filter("pessoaFisica.cpf", "string", null, "eq"));
			params.getFilters().add(new Filter("pessoaJuridica.cnpj", "string", null, "eq"));
		}
				
		SearchSeven searchSeven = new SearchSeven(params);

		if(filter.getValue() != null){
			searchSeven.addFilterOr(com.googlecode.genericdao.search.Filter.like("pessoaFisica.cpf", "%"+ filter.getValue() +"%"),
									com.googlecode.genericdao.search.Filter.like("pessoaJuridica.cnpj", "%"+ filter.getValue() +"%"));
		}
		
		searchSeven.setResultMode(Search.RESULT_MAP);
		
		SearchResult searchResult = searchAndCount(searchSeven);
		
		for(int i = 0; i < searchResult.getResult().size(); i++){
			HashMap<String, Object> pessoa = (HashMap<String, Object>) searchResult.getResult().get(i);
			
			if(pessoa.get("pessoaJuridica.cnpj") != null){
				pessoa.put("cpfCnpj", UtilString.formatarString((String) pessoa.get("pessoaJuridica.cnpj"), "##.###.###/####-##"));
				
			}else if(pessoa.get("pessoaFisica.cpf") != null){
				pessoa.put("cpfCnpj", UtilString.formatarString((String) pessoa.get("pessoaFisica.cpf"), "###.###.###-##"));
			}
		}
		
		return searchResult;
	}

}
