package br.com.centralit.api.model;

import java.math.BigDecimal;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

import org.hibernate.envers.Audited;

import com.fasterxml.jackson.annotation.JsonView;

import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObject;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 29/05/2015 - 15:41:54
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Audited
@Entity
public class UnidadeMedida extends PersistentObject {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = -7573208249312737668L;
	
	@Transient
	private final Long TIPO_NUMERICO_INTEIRO = new Long (1l);
	@Transient
	private final Long TIPO_NUMERICO_DECIMAL = new Long (2l);
	
	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	@Column(length = 30)
	@JsonView({ Views.GenericView.class })
	private String codigo;

	@JsonView({ Views.GenericView.class })
	private String sigla;

	@JsonView({ Views.GenericView.class })
	private String descricao;

	/** Atributo tipoNumerico. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ Views.GenericView.class })
	private Dominio tipoNumerico;

	@Column(name = "quantidadePadrao")
	@JsonView({ Views.GenericView.class })
	private BigDecimal quantidadePadrao;

	@Column(name = "isUnidadePadrao")
	@JsonView({ Views.GenericView.class })
	private Boolean isUnidadePadrao;
	
	@Transient
	@JsonView({ Views.GenericView.class })
	private BigDecimal quantidadeInteiro;

	@Transient
	@JsonView({ Views.GenericView.class })
	private BigDecimal quantidadeDecimal;
	
	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public UnidadeMedida() {

		super();
	}

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param tipoNumerico
	 * @param tipoUnidadeMedida
	 */
	public UnidadeMedida( String sigla, Dominio tipoNumerico, String descricao, String codigo ) {

		this.tipoNumerico = tipoNumerico;

		this.descricao = descricao;

		this.sigla = sigla;

		this.codigo = codigo;
	}

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>tipoNumerico</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoNumerico() {

		return tipoNumerico;
	}

	/**
	 * Define o valor do atributo <code>tipoNumerico</code>.
	 *
	 * @param tipoNumerico
	 */
	public void setTipoNumerico(Dominio tipoNumerico) {

		this.tipoNumerico = tipoNumerico;
	}

	/**
	 * Retorna o valor do atributo <code>descricao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricao() {

		return descricao;
	}

	/**
	 * Define o valor do atributo <code>descricao</code>.
	 *
	 * @param descricao
	 */
	public void setDescricao(String descricao) {

		this.descricao = descricao;
	}

	/**
	 * Retorna o valor do atributo <code>sigla</code>
	 *
	 * @return <code>String</code>
	 */
	public String getSigla() {

		return sigla;
	}

	/**
	 * Define o valor do atributo <code>sigla</code>.
	 *
	 * @param sigla
	 */
	public void setSigla(String sigla) {

		this.sigla = sigla;
	}

	/**
	 * Retorna o valor do atributo <code>codigo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCodigo() {

		return codigo;
	}

	/**
	 * Define o valor do atributo <code>codigo</code>.
	 *
	 * @param codigo
	 */
	public void setCodigo(String codigo) {

		this.codigo = codigo;
	}

	public BigDecimal getQuantidadePadrao() {
		return quantidadePadrao;
	}

	public void setQuantidadePadrao(BigDecimal quantidadePadrao) {
		this.quantidadePadrao = quantidadePadrao;
	}

	public Boolean getIsUnidadePadrao() {
		return isUnidadePadrao;
	}

	public void setIsUnidadePadrao(Boolean isUnidadePadrao) {
		this.isUnidadePadrao = isUnidadePadrao;
	}
	
	public BigDecimal getQuantidadeInteiro() {
		return quantidadePadrao;
	}

	public void setQuantidadeInteiro(BigDecimal quantidadeInteiro) {
		if(this.tipoNumerico != null && this.tipoNumerico.getCodigo().equals(TIPO_NUMERICO_INTEIRO)){
			this.quantidadePadrao = quantidadeInteiro;
		}
	}
	
	public BigDecimal getQuantidadeDecimal() {
		return quantidadePadrao;
	}

	public void setQuantidadeDecimal(BigDecimal quantidadeDecimal) {
		if(this.tipoNumerico != null && this.tipoNumerico.getCodigo().equals(TIPO_NUMERICO_DECIMAL)){
			this.quantidadePadrao = quantidadeDecimal;
		}
	}
	
}
