package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.AlcadaDao;
import br.com.centralit.api.model.Alcada;
import br.com.centralit.api.service.AlcadaNivelAutoridadeService;
import br.com.centralit.api.service.AlcadaService;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 23/06/2015 - 11:19:30
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.ribeiro@centralit.com.br">lucas.ribeiro@centralit.com.br</a>)
 *	
 */
@Service("alcadaService")
public class AlcadaServiceImpl extends GenericServiceImpl<Alcada, Long> implements AlcadaService {

	/** Atributo alcadaNivelAutoridadeService. */
	@Autowired
	private AlcadaNivelAutoridadeService alcadaNivelAutoridadeService;
	
    private AlcadaDao alcadaDao;

    @Autowired
	public AlcadaServiceImpl(AlcadaDao alcadaDao,  @Qualifier("alcadaValidator") Validator validator) {
    	this.dao = alcadaDao;
        this.alcadaDao = alcadaDao;
    	this.validator = validator;
	}
    
    /**
     *
     */
    @Override
    public Alcada save(Alcada alcada) {
    
    	// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {
			this.validarEntidade(alcada, this.validator);
		}

		// MONTA OS DADOS DO OBJETO ALCADANIVELAUTORIDADE
		this.alcadaNivelAutoridadeService.montarAlcadaNivelAutoridade(alcada);

		// SALVA A TRANSFERÊNCIA;
		return super.save(alcada);
    }
    
    /**
     *
     */
    @Override
    public Alcada merge(Alcada alcada) {
    
    	if (UtilObjeto.isReferencia(this.validator)) {
			this.validarEntidade(alcada, this.validator);
		}

		// MONTA OS DADOS DO OBJETO ALCADANIVELAUTORIDADE
		this.alcadaNivelAutoridadeService.montarAlcadaNivelAutoridade(alcada);

		// SALVA A TRANSFERÊNCIA
		Alcada entitySaved = (Alcada) this.alcadaDao.merge(alcada);
		
    	return entitySaved;
    }

	@Override
	public Collection<Alcada> listarAlcadas(String nome) {

		Collection<Alcada> listaAlcadas = this.alcadaDao.listarAlcadas(nome);
		return UtilColecao.isVazio(listaAlcadas) ? new ArrayList<Alcada>() : listaAlcadas;
	}

	@Override
	public Alcada findByIdentificaodr(String nome) {
		return alcadaDao.findByIdentificador(nome);
	}

	@Override
	public boolean existeAlcadaVinculadaAoNivelAutoridade(long idNivelAutoridade) {
		
		return this.alcadaDao.existeAlcadaVinculadaAoNivelAutoridade(idNivelAutoridade);
	}
}
