package br.com.centralit.api.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.CentroCustoDao;
import br.com.centralit.api.model.CentroCusto;
import br.com.centralit.api.service.CentroCustoService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

@Service("centroCustoService")
public class CentroCustoServiceImpl extends GenericServiceImpl<CentroCusto, Long> implements CentroCustoService {

    private CentroCustoDao centroCustoDao;
    
    @Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

    @Autowired
	public CentroCustoServiceImpl(CentroCustoDao centroCustoDao,  @Qualifier("centroCustoValidator") Validator validator) {

    	this.dao = centroCustoDao;

        this.centroCustoDao = centroCustoDao;

    	this.validator = validator;

	}
    
    @Override
    public CentroCusto save(CentroCusto entity) {
    
    	this.validarEntidade(entity, this.validator);

		entity = super.save(entity);

		configurarCodigo(entity);
		
		return entity;
    }
    
    @Override
    public CentroCusto merge(CentroCusto entity) {
    
    	this.validarEntidade(entity, this.validator);

    	if(UtilObjeto.isReferencia(entity.getCentroCustoParent()) && UtilObjeto.isReferencia(entity.getCentroCustoParent().getCentroCustoParent()) && entity.getId().equals(entity.getCentroCustoParent().getCentroCustoParent().getId())){
    		throw new BusinessException("CORPORATIVO.VALIDACAO.CENTRO_CUSTO_VINCULADO", CodigoErro.REGRA_NEGOCIO.getValue(), "");
    	}
    	
		return super.merge(entity);
    }
    
	/**
	 *
	 * Método responsável por settar o atributo codigo.
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param entity
	 */
	private void configurarCodigo(CentroCusto entity) {

		CentroCusto ultimoRegistro = (CentroCusto) this.centroCustoDao.buscarUltimoRegistroComOrdenadoParametrizada("codigo", entity.getOrganizacao().getId());

		Long sequencial = null;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo() == null ? null : Long.parseLong(ultimoRegistro.getCodigo());
		}
		String codigo = this.configuracaoParametroSistemaService.gerarNumeroIdentificacaoSeguencial(sequencial);

		entity.setCodigo(codigo);
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por listar centros de custo por organizacao do usuario logado
	 *
	 * @author iago.almeida
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idCentroCusto
	 * @return List<CentroCusto>
	 */
	@Override
	public List<CentroCusto> findAutoCompleteCentroCustoByOrganizacao(String codigo, Long idOrganizacao, Long idCentroCusto) {

		return this.centroCustoDao.findAutoCompleteCentroCustoByOrganizacao(codigo, idOrganizacao, idCentroCusto);

	}

}
