package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.CentroResultadoResponsavelDao;
import br.com.centralit.api.model.CentroResultado;
import br.com.centralit.api.model.CentroResultadoResponsavel;
import br.com.centralit.api.model.CentroResultadoResponsavelAlcada;
import br.com.centralit.api.model.Colaborador;
import br.com.centralit.api.service.AlcadaService;
import br.com.centralit.api.service.CentroResultadoResponsavelAlcadaService;
import br.com.centralit.api.service.CentroResultadoResponsavelService;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 30/06/2015 - 14:07:09
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.ribeiro@centralit.com.br">lucas.ribeiro@centralit.com.br</a>)
 *	
 */
@Service("centroResultadoResponsavelService")
public class CentroResultadoResponsavelServiceImpl extends GenericServiceImpl<CentroResultadoResponsavel, Long> implements CentroResultadoResponsavelService {
	
	/** Constante VINCULACAO_USUARIO. */
	private static final String SUBSTITUTO_NAO_PERMITIDO = "MSG.SUBSTITUTO_NAO_PERMITIDO";

	/** Atributo alcadaService. */
	@Autowired
	private AlcadaService alcadaService;
	
	/** Atributo alcadaService. */
	@Autowired
	private ColaboradorService colaboradorService;
	
	/** Atributo alcadaService. */
	@Autowired
	private CentroResultadoResponsavelAlcadaService centroResultadoResponsavelAlcadaService;
	
    private CentroResultadoResponsavelDao centroResultadoAlcadaResponsavelDao;

    @Autowired
	public CentroResultadoResponsavelServiceImpl(CentroResultadoResponsavelDao centroResultadoAlcadaResponsavelDao,  @Qualifier("centroResultadoResponsavelValidator") Validator validator) {
    	this.dao = centroResultadoAlcadaResponsavelDao;
        this.centroResultadoAlcadaResponsavelDao = centroResultadoAlcadaResponsavelDao;
    	this.validator = validator;
	}

	/**
	 *
	 */
	@Override
	public void montarCentroResultadoResponsavel(CentroResultado centroResultado) {

		// PERCORRE A LISTA DE RESPONSAVEIS
		for(CentroResultadoResponsavel centroResultadoResponsavel : centroResultado.getResponsaveis()) {			
			// CARREGA O OBJETO centroResultado
			centroResultadoResponsavel.setCentroResultado(centroResultado);
			centroResultadoResponsavel.setResponsavel(this.colaboradorService.getReference(centroResultadoResponsavel.getResponsavel().getId()));
			centroResultadoResponsavelAlcadaService.montarCentroResultadoResponsavelAlcada(centroResultadoResponsavel);
		}
	}

	@Override
	public List<CentroResultadoResponsavel> buscaHistoricoResponsaveis(Long idCentroResultado) {
		
		return centroResultadoAlcadaResponsavelDao.buscaHistoricoResponsaveis(idCentroResultado);
	}    
	
	@Override
	public Collection<CentroResultadoResponsavel> findPorIdCentroResultado(Long idCentroResultado) {
		
		return centroResultadoAlcadaResponsavelDao.findPorIdCentroResultado(idCentroResultado);
	}

	@Override
	public void substitui(CentroResultadoResponsavel centroResultadoResponsavel, Colaborador substituto) {
		Collection<CentroResultadoResponsavel> responsaveis = this.findPorIdCentroResultado(centroResultadoResponsavel.getCentroResultado().getId());
		for (CentroResultadoResponsavel centroResultadoResp : responsaveis) {
			if (centroResultadoResp.getResponsavel().getId().equals(substituto.getId())) {
				throw new BusinessException(SUBSTITUTO_NAO_PERMITIDO, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		}
		
		this.remove(centroResultadoResponsavel);
		
		CentroResultadoResponsavel novoResponsavel = new CentroResultadoResponsavel();
		novoResponsavel.setCentroResultado(centroResultadoResponsavel.getCentroResultado());
		novoResponsavel.setResponsavel(substituto);
	
		Collection<CentroResultadoResponsavelAlcada> alcadas = new ArrayList<CentroResultadoResponsavelAlcada>();
		for (CentroResultadoResponsavelAlcada centroResultadoResponsavelAlcada : centroResultadoResponsavel.getAlcadas()) {
			CentroResultadoResponsavelAlcada novaAlcada = new CentroResultadoResponsavelAlcada();
			novaAlcada.setCentroResultadoResponsavel(novoResponsavel);
			novaAlcada.setAlcada(centroResultadoResponsavelAlcada.getAlcada());
			alcadas.add(novaAlcada);
		}
		
		novoResponsavel.setAlcadas(alcadas);
		this.save(novoResponsavel);
	}
	
}
