package br.com.centralit.api.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.CentroResultadoDao;
import br.com.centralit.api.model.CentroResultado;
import br.com.centralit.api.model.CentroResultadoResponsavel;
import br.com.centralit.api.service.CentroResultadoResponsavelService;
import br.com.centralit.api.service.CentroResultadoService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 30/06/2015 - 14:09:22
 * 
 * @version 1.0.0
 * 
 * @author lucas.ribeiro - (<a href="mailto:lucas.ribeiro@centralit.com.br">lucas.ribeiro@centralit.com.br</a>)
 * 
 */
@Service("centroResultadoService")
public class CentroResultadoServiceImpl extends GenericServiceImpl<CentroResultado, Long> implements CentroResultadoService {

	/** Constante VALIDACAO_CODIGO_IGUAL. */
	private static final String VALIDACAO_CODIGO_IGUAL = "VALIDACAO.CENTRO_RESULTADO_CODIGO_IGUAL";
	
	/** Constante VALIDACAO_NOME_IGUAL. */
	private static final String VALIDACAO_NOME_IGUAL = "VALIDACAO.CENTRO_RESULTADO_NOME_IGUAL";

	/** Atributo centroResultadoAlcadaResponsavelService. */
	@Autowired
	private CentroResultadoResponsavelService centroResultadoResponsavelService;

	private CentroResultadoDao centroResultadoDao;

	@Autowired
	public CentroResultadoServiceImpl( CentroResultadoDao centroResultadoDao, @Qualifier("centroResultadoValidator") Validator validator ) {

		this.dao = centroResultadoDao;

		this.centroResultadoDao = centroResultadoDao;

		this.validator = validator;

	}

	public CentroResultado save(CentroResultado centroResultado) {

		// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {
			this.validarEntidade(centroResultado, this.validator);
		}

		// VERIFICA SE O CODIGO E UNICO
		this.codigoIsUnico(centroResultado);

		// VERIFICA SE O NOME E UNICO
		this.nomeIsUnico(centroResultado);

		if (centroResultado.getResponsaveis() != null && !centroResultado.getResponsaveis().isEmpty())
			montaCentroResultado(centroResultado);

		return super.save(centroResultado);
	}

	/**
     *
     */
	@Override
	public CentroResultado merge(CentroResultado centroResultado) {

		// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {
			this.validarEntidade(centroResultado, this.validator);
		}

		// VERIFICA SE O CODIGO E UNICO
		this.codigoIsUnico(centroResultado);

		// VERIFICA SE O NOME E UNICO
		this.nomeIsUnico(centroResultado);

		if (centroResultado.getResponsaveis() != null && !centroResultado.getResponsaveis().isEmpty())
			montaCentroResultado(centroResultado);
		
		if (!UtilObjeto.isReferencia(centroResultado.getCentroResultadoParent())){
			
			if (centroResultado.getResponsaveis() != null && !centroResultado.getResponsaveis().isEmpty()){
				for (CentroResultadoResponsavel cr : centroResultado.getResponsaveis()){
					this.centroResultadoResponsavelService.removeById(cr.getId());
				}
			}
				

		}

		return super.merge(centroResultado);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o codigo do centro de resultado e unico
	 * 
	 * @author renato.jesus
	 * 
	 * @param centroResultado
	 */
	private void codigoIsUnico(CentroResultado centroResultado) {

		if (UtilObjeto.isReferencia(centroResultado) && UtilObjeto.isReferencia(centroResultado.getId())) {

			CentroResultado centroResultadoBase = this.find(centroResultado.getId());

			if (!centroResultado.getCodigo().equals(centroResultadoBase.getCodigo()) && this.centroResultadoDao.codigoIsUnico(centroResultado.getCodigo())) {
				throw new BusinessException(VALIDACAO_CODIGO_IGUAL, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		} else if (this.centroResultadoDao.codigoIsUnico(centroResultado.getCodigo())) {

			throw new BusinessException(VALIDACAO_CODIGO_IGUAL, CodigoErro.REGRA_NEGOCIO.getValue());
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se o nome do centro de resultado e unico
	 * 
	 * @author renato.jesus
	 * 
	 * @param centroResultado
	 */
	private void nomeIsUnico(CentroResultado centroResultado) {

		if (UtilObjeto.isReferencia(centroResultado) && UtilObjeto.isReferencia(centroResultado.getId())) {

			CentroResultado centroResultadoBase = this.find(centroResultado.getId());

			if (!centroResultado.getNome().equals(centroResultadoBase.getNome()) && this.centroResultadoDao.nomeIsUnico(centroResultado.getNome())) {
				throw new BusinessException(VALIDACAO_NOME_IGUAL, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		} else if (this.centroResultadoDao.nomeIsUnico(centroResultado.getNome())) {

			throw new BusinessException(VALIDACAO_NOME_IGUAL, CodigoErro.REGRA_NEGOCIO.getValue());
		}
	}

	private void montaCentroResultado(CentroResultado centroResultado) {

		// MONTA OS DADOS DO OBJETO CENTRORESULTADOALCADARESPONSAVEL
		this.centroResultadoResponsavelService.montarCentroResultadoResponsavel(centroResultado);
	}	

	@Override
	public CentroResultado getReference(Long id) {

		CentroResultado centroResultado = super.getReference(id);

		centroResultado.setResponsaveis(this.centroResultadoResponsavelService.findPorIdCentroResultado(id));

		return centroResultado;
	}

	@Override
	public CentroResultado findByCodigo(String codigo) {

		return centroResultadoDao.findByCodigo(codigo);
	}

	@Override
	public CentroResultado findByCodigoSistemaOrigem(String codigo) {
		
		return centroResultadoDao.findByCodigoSistemaOrigem(codigo);
	}

}
