package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.CentroResultadoSubstituicaoDao;
import br.com.centralit.api.model.Alcada;
import br.com.centralit.api.model.CentroResultadoResponsavel;
import br.com.centralit.api.model.CentroResultadoResponsavelAlcada;
import br.com.centralit.api.model.CentroResultadoSubstituicao;
import br.com.centralit.api.model.Colaborador;
import br.com.centralit.api.service.CentroResultadoResponsavelService;
import br.com.centralit.api.service.CentroResultadoSubstituicaoService;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.framework.esi.enumerated.VariableTypeEnum;
import br.com.centralit.framework.esi.environment.EnvironmentVariable;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;
import br.com.centralit.framework.util.UtilString;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 30/06/2015 - 14:07:09
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.ribeiro@centralit.com.br">lucas.ribeiro@centralit.com.br</a>)
 *	
 */
@Service("centroResultadoSubstituicaoService")
public class CentroResultadoSubstituicaoServiceImpl extends GenericServiceImpl<CentroResultadoSubstituicao, Long> implements CentroResultadoSubstituicaoService {

	@Autowired
	private CentroResultadoResponsavelService centroResultadoResponsavelService;
	
	@Autowired
	private ColaboradorService colaboradorService;
	
	private CentroResultadoSubstituicaoDao centroResultadoSubstituicaoDao;

    @Autowired
	public CentroResultadoSubstituicaoServiceImpl(CentroResultadoSubstituicaoDao centroResultadoSubstituicaoDao,  @Qualifier("centroResultadoSubstituicaoValidator") Validator validator) {
    	this.dao = centroResultadoSubstituicaoDao;
        this.centroResultadoSubstituicaoDao = centroResultadoSubstituicaoDao;
    	this.validator = validator;
	}


	@Override
	public Collection<CentroResultadoSubstituicao> findPorIdCentroResultado(Long idCentroResultado) {
		
		return centroResultadoSubstituicaoDao.findPorIdCentroResultado(idCentroResultado);
	}
	
	@Override
	public CentroResultadoSubstituicao save(CentroResultadoSubstituicao centroResultadoSubstituicao) {
		if (UtilObjeto.isReferencia(this.validator)) {
			this.validarEntidade(centroResultadoSubstituicao, this.validator);
		}
	
		centroResultadoSubstituicao.setSubstituto(colaboradorService.getReference(centroResultadoSubstituicao.getSubstituto().getId()));
		
		CentroResultadoResponsavel centroResultadoResponsavel = centroResultadoResponsavelService.getReference(centroResultadoSubstituicao.getCentroResultadoResponsavel().getId());
		centroResultadoResponsavelService.substitui(centroResultadoResponsavel, centroResultadoSubstituicao.getSubstituto());

		centroResultadoSubstituicao.setCentroResultadoResponsavel(centroResultadoResponsavel);
		centroResultadoSubstituicao = super.save(centroResultadoSubstituicao);
		
		for (CentroResultadoResponsavelAlcada centroResultadoResponsavelAlcada : centroResultadoResponsavel.getAlcadas()) {
			this.disparaFluxoDelegacao(centroResultadoResponsavelAlcada.getAlcada(), centroResultadoResponsavel.getResponsavel(), centroResultadoSubstituicao.getSubstituto());
		}
		
		return centroResultadoSubstituicao;
	}
	
	private void disparaFluxoDelegacao(Alcada alcada, Colaborador colaboradorResponsavel, Colaborador colaboradorSubstituto) {
		if (!UtilString.isNullOrEmpty(alcada.getFluxoDelegacao())) {
			Usuario usuario = colaboradorResponsavel.getPessoa().getUsuario();
			if (!UtilObjeto.isReferencia(usuario)) {
				throw new BusinessException("MSG.USUARIO_NAO_ASSOCIADO_RESPONSAVEL", CodigoErro.REGRA_NEGOCIO.getValue());
			}
			String responsavel = usuario.getUsername();

			usuario = colaboradorSubstituto.getPessoa().getUsuario();
			if (!UtilObjeto.isReferencia(usuario)) {
				throw new BusinessException("MSG.USUARIO_NAO_ASSOCIADO_DELEGACAO", CodigoErro.REGRA_NEGOCIO.getValue());
			}
			String delegado = usuario.getUsername();

			List<EnvironmentVariable> variaveis = new ArrayList<EnvironmentVariable>();
			variaveis.add(new EnvironmentVariable("alcada", VariableTypeEnum.TEXT, alcada.getIdentificador(), false));
			variaveis.add(new EnvironmentVariable("responsavel", VariableTypeEnum.TEXT, responsavel, false));
			variaveis.add(new EnvironmentVariable("delegado", VariableTypeEnum.TEXT, delegado, false));
			variaveis.add(new EnvironmentVariable("abrangencia", VariableTypeEnum.TEXT, "NOVAS_E_ANDAMENTO", false));
			variaveis.add(new EnvironmentVariable("requisicoes", VariableTypeEnum.JAVAOBJECT, new ArrayList<String>(), false));
			
			this.startProcess(alcada.getFluxoDelegacao(), variaveis);
		}
	}
}
