package br.com.centralit.api.service.impl;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.correios.AtendeClienteService;
import br.com.centralit.api.correios.EnderecoERP;
import br.com.centralit.api.correios.SQLException_Exception;
import br.com.centralit.api.correios.SigepClienteException;
import br.com.centralit.api.dao.EnderecoDao;
import br.com.centralit.api.model.Bairro;
import br.com.centralit.api.model.Cidade;
import br.com.centralit.api.model.Endereco;
import br.com.centralit.api.model.Estado;
import br.com.centralit.api.service.BairroService;
import br.com.centralit.api.service.CidadeService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EnderecoService;
import br.com.centralit.api.service.EstadoService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 05/12/2014 - 11:11:25
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Service("enderecoService")
public class EnderecoServiceImpl extends GenericServiceImpl<Endereco, Long> implements EnderecoService {

	/** Atributo enderecoDao. */
	private EnderecoDao enderecoDao;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo bairroService. */
	@Autowired
	private BairroService bairroService;
	
	/** Atributo cidadeService. */
	@Autowired
	private CidadeService cidadeService;
	
	/** Atributo cidadeService. */
	@Autowired
	private EstadoService estadoService;

	/** Atributo configuracaoParametroSistemaService. */
	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;
	
	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param enderecoDao
	 * @param validator
	 */
	@Autowired 
	public EnderecoServiceImpl( EnderecoDao enderecoDao, @Qualifier("enderecoValidator") Validator validator ) {

		this.dao = enderecoDao;

		this.enderecoDao = enderecoDao;

		this.validator = validator;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">563</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por salvar endereço
	 *
	 * @author iago.almeida
	 *
	 * @param endereco
	 * @return <Endereco>
	 */
	@Override
	public Endereco save(Endereco endereco) {

		endereco.setBairro((Bairro) ( ( endereco.getBairro() != null && endereco.getBairro().getId() != null ) ? this.bairroService.getReference(endereco.getBairro().getId()) : null ));
		endereco.setCidade(( endereco.getBairro() != null && endereco.getBairro().getId() != null ) ? endereco.getBairro().getCidade() : null);
		endereco.setDominioTipoEndereco((Dominio) ( ( endereco.getDominioTipoEndereco() != null && endereco.getDominioTipoEndereco().getId() != null ) ? this.dominioService.getReference(endereco.getDominioTipoEndereco().getId()) : null ));

		// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {

			this.validarEntidade(endereco, this.validator);
		}

		// VERIFICA SE O ENDERECO JÁ TEM CÓDIGO, SENÃO GERA E SALVA ENTIDADE
		if(endereco.getCodigo() != null){

			return super.save(endereco);

		} else {

			Endereco entitySaved = super.save(endereco);

			this.configurarCodigo(entitySaved);

			return entitySaved;

		}

	}


	/**
	 *
	 * Método responsável por atualizar o endereço
	 *
	 * @author gilberto.nery
	 *
	 * @param endereco
	 * @return <Endereco>
	 *
	 */
	@Override
	public Endereco merge(Endereco endereco) {

		endereco.setBairro((Bairro) ( ( endereco.getBairro() != null && endereco.getBairro().getId() != null ) ? this.bairroService.getReference(endereco.getBairro().getId()) : null ));
		endereco.setCidade(( endereco.getBairro() != null && endereco.getBairro().getId() != null ) ? endereco.getBairro().getCidade() : null);
		endereco.setDominioTipoEndereco((Dominio) ( ( endereco.getDominioTipoEndereco() != null && endereco.getDominioTipoEndereco().getId() != null ) ? this.dominioService.getReference(endereco.getDominioTipoEndereco().getId()) : null ));

		// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {

			this.validarEntidade(endereco, this.validator);
		}

		return super.merge(endereco);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar a entidade <code>Endereco</code>
	 *
	 * @author iago.almeida
	 *
	 * @param nome
	 * @return Collection<Endereco>
	 */
	public Collection<Endereco> listarEndereco(String nome) {

		return enderecoDao.listarEndereco(nome);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por inativar o endereco setando dataInativo.
	 *
	 * @author iago.almeida
	 *
	 * @param enderecoId
	 * @return
	 */
	@Override
	public boolean removeById(Long enderecoId) {

		if(this.enderecoDao.existeEnderecoVinculadoALocalizacao(enderecoId)){

			throw new BusinessException("PORTAL.MSG.MN020", CodigoErro.REGRA_NEGOCIO.getValue());

		}

		return this.dao.removeById(enderecoId);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar os enderecos através do id da pessoa
	 *
	 * @author rogerio.costa
	 *
	 * @param idPessoa
	 *
	 * @return Collection<Parceiro>
	 */
	public Collection<Endereco> findPorIdPessoa(Long idPessoa) {

		return this.enderecoDao.findPorIdPessoa(idPessoa);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean existeBairroVinculadoAEndereco(Long bairroId) {

		return this.enderecoDao.existeBairroVinculadoAEndereco(bairroId);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por settar o atributo codigo.
	 *
	 * @author iago.almeida
	 *
	 * @param entity
	 */
	private void configurarCodigo(Endereco entity) {

		Endereco ultimoRegistro = (Endereco) this.enderecoDao.buscarUltimoRegistroComOrdenadoParametrizada("codigo");

		Long sequencial = null;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo() == null ? null : Long.parseLong(ultimoRegistro.getCodigo());
		}
		String codigo = this.configuracaoParametroSistemaService.gerarNumeroIdentificacaoSeguencial(sequencial);

		entity.setCodigo(codigo);
	}
	
	@Override
	public String buscaUltimoCodigoEndereco(){
		Endereco ultimoRegistro = (Endereco) this.enderecoDao.buscarUltimoRegistroComOrdenadoParametrizada("codigo");
		
		Long sequencial = 0L;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo() == null ? null : Long.parseLong(ultimoRegistro.getCodigo());
		}
		
		return sequencial.toString();
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por buscar a entidade <code>Endereco</code> através do CEP
	 *
	 * @author luis.camargo
	 *
	 * @param nome
	 * @return <Endereco>
	 */
	@Override
	public Endereco buscarEnderecoWSCorreio(String cep) {
		Endereco endereco = null;
		try {
			EnderecoERP enderecoCorreios = new AtendeClienteService().getAtendeClientePort().consultaCEP(cep);
			endereco = new Endereco();
			endereco.setCep(enderecoCorreios.getCep());
			endereco.setLogradouro(enderecoCorreios.getEnd());
			endereco.setComplemento(enderecoCorreios.getComplemento());

			Estado estado = estadoService.buscarEstadoPorSigla(enderecoCorreios.getUf());
			Cidade cidade = cidadeService.buscarCidade(enderecoCorreios.getCidade(), estado.getId());
			
			Bairro bairro = null;
			if (cidade != null) {
				bairro = bairroService.buscarBairro(enderecoCorreios.getBairro(), cidade.getId());
			} else {
				cidade  = new Cidade();
				cidade.setEstado(estado);
			}
			
			endereco.setCidade(cidade);
			endereco.setBairro(bairro);
		} catch (SigepClienteException ex) {
			throw new BusinessException(ex.getFaultInfo(), CodigoErro.REGRA_NEGOCIO.getValue());
		} catch (SQLException_Exception e) {
			throw new BusinessException("MSG.SISTEMA_CORREIO_INDISPONIVEL", CodigoErro.REGRA_NEGOCIO.getValue());
		} 
		
		return endereco;
	}

}
