package br.com.centralit.api.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.TerceirizadoDao;
import br.com.centralit.api.model.Terceirizado;
import br.com.centralit.api.service.ClasseParceiroService;
import br.com.centralit.api.service.TerceirizadoService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 15/12/2015 - 15:14:13
 *
 * @version 1.0.0
 *
 * @author rogerio.cassimiro
 *	
 */
@Service("terceirizadoService")
public class TerceirizadoServiceImp extends GenericServiceImpl<Terceirizado, Long> implements TerceirizadoService {

	private TerceirizadoDao terceirizadoDao;

	/** Atributo classeParceiroService. */
	@Autowired
	private ClasseParceiroService classeParceiroService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param portadorDao
	 */
	@Autowired
	public TerceirizadoServiceImp( TerceirizadoDao terceirizadoDao, @Qualifier("terceirizadoValidator") Validator validator ) {

		this.dao = terceirizadoDao;

		this.terceirizadoDao = terceirizadoDao;
		
		this.validator = validator;

	}

	@Override
	public Terceirizado save(Terceirizado entity) {
		
		entity.setClasseParceiro(this.classeParceiroService.obterPorCodigoDominioTipParceiro(Dominio.TIPO_PARCEIRO_TERCEIRIZADO));

		this.validarEntidade(entity, this.validator);

		return super.save(entity);
	}
	
	/**
	 * Método responsável por obter permissão para assinatura de documento
	 * @author rogerio.cassimiro
	 * @return {@link Boolean}
	 */
	@Override
	public boolean getPermissaoAssinaturaDocumento() {
		Usuario usuario = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		return this.terceirizadoDao.getPermissaoAssinaturaDocumento(usuario.getId());
	}

}
