package br.com.centralit.api.service.impl;

import java.math.BigDecimal;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.UnidadeMedidaDao;
import br.com.centralit.api.model.UnidadeMedida;
import br.com.centralit.api.service.UnidadeMedidaService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

@Service("unidadeMedidaService")
public class UnidadeMedidaServiceImpl extends GenericServiceImpl<UnidadeMedida, Long> implements UnidadeMedidaService {

    private UnidadeMedidaDao unidadeMedidaDao;

    @Autowired
	public UnidadeMedidaServiceImpl(UnidadeMedidaDao unidadeMedidaDao,  @Qualifier("unidadeMedidaValidator") Validator validator) {
    	this.dao = unidadeMedidaDao;
        this.unidadeMedidaDao = unidadeMedidaDao;
    	this.validator = validator;
	}
    
    @Override
    public UnidadeMedida save(UnidadeMedida entity) {
    	
    	if(!UtilObjeto.isReferencia(entity.getIsUnidadePadrao())){
    		entity.setIsUnidadePadrao(Boolean.FALSE);
    	}
    	
    	if(UtilObjeto.isReferencia(entity.getQuantidadePadrao()) && entity.getQuantidadePadrao().equals(BigDecimal.ZERO)){
    		throw new BusinessException("CORPORATIVO.MSG.QTDE_VALOR_PADRAO_NAO_PERMITIDO", CodigoErro.REGRA_NEGOCIO.getValue(), "");
    	}
    	return super.save(entity);
    }
 
    @Override
    public UnidadeMedida merge(UnidadeMedida entity) {
    
    	if(!UtilObjeto.isReferencia(entity.getIsUnidadePadrao())){
    		entity.setIsUnidadePadrao(Boolean.FALSE);
    	}
    	
    	if(UtilObjeto.isReferencia(entity.getQuantidadePadrao()) && entity.getQuantidadePadrao().equals(BigDecimal.ZERO)){
    		throw new BusinessException("CORPORATIVO.MSG.QTDE_VALOR_PADRAO_NAO_PERMITIDO", CodigoErro.REGRA_NEGOCIO.getValue(), "");
    	}
    	// TODO Auto-generated method stub
    	return super.merge(entity);
    }
}
