package br.com.centralit.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.framework.json.ViewsTabelasCorp;
import br.com.centralit.api.model.CentroCusto;
import br.com.centralit.api.service.CentroCustoService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views.GenericView;
import br.com.centralit.framework.view.ResultResponseVH;

@Controller
@RequestMapping("/rest/centroCusto")
public class CentroCustoController extends GenericController<CentroCusto>{

	/** Atributo centroCustoService. */
	private CentroCustoService centroCustoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contabilService
	 */
	@Autowired
	public CentroCustoController(CentroCustoService centroCustoService) {

		super(centroCustoService);

		this.centroCustoService = centroCustoService;

	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por listar centros de custo por organizacao do usuario logado
	 *
	 * @author iago.almeida
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @return List<CentroCusto>
	 */
	@RequestMapping(value = "/findAutoCompleteCentroCustoByOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAutoCompleteCentroCustoByOrganizacao(@RequestParam(value = "value") String value, @RequestParam(value = "idOrganizacao") Long idOrganizacao,@RequestParam(value = "idCentroCusto", required = false)  Long idCentroCusto) {

		ResultResponseVH<List<CentroCusto>> resultResponseVH = new ResultResponseVH<List<CentroCusto>>(centroCustoService.findAutoCompleteCentroCustoByOrganizacao(value, idOrganizacao, idCentroCusto));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, getEditView());

		return responseBody;
	}
	
	@Override
	public Class<? extends GenericView> getEditView() {
		return ViewsTabelasCorp.CentroCustoEditView.class;
	}
	
	@Override
	public Class<? extends GenericView> getListView() {
		return ViewsTabelasCorp.CentroCustoListView.class;
	}
}
