package br.com.centralit.listener;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

import br.com.centralit.api.model.ClasseParceiro;
import br.com.centralit.api.service.ClasseParceiroService;
import br.com.centralit.api.service.DefaultFileService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.framework.model.DefaultFile;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Internacionalizacao;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.UtilStartup;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p> Startup listener criado para fazer insert de recurso essencial ao funcionamento do sistema, na inicializacao do mesmo
 * ClasseParceiro, sem ClasseParceiro a pessoa da erro quando inserido um parceiro.
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 11/09/2015 - 11:09:14
 *
 * @version 1.0.0
 *
 * @author juliana.barbosa
 *
 */
@Component
public class StartupListenerTabelasCorp extends UtilStartup implements ApplicationListener<ContextRefreshedEvent> {

	@Autowired
	private DominioService dominioService;
	
	@Autowired
	private ClasseParceiroService classeParceiroService;
	
	@Autowired
	private DefaultFileService defaultFileService;
	
	@Autowired
	private InternacionalizacaoService internacionalizacaoService;
	
	@Autowired
	private ModuloService moduloService;
	
	@Override
	public void onApplicationEvent(final ContextRefreshedEvent event) {
		criarDominios();
		criarClasseParceiro();
		criarDefaultFile();
		gerarInternacionalizacao();
	}
	
	/**
	 * Método responsável por criar os dominios referentes ao modulo de tabelas corp
	 * @author gilberto.nery
	 */
	private void criarDominios() {

		List<Dominio> list = new ArrayList<Dominio>();
		list.add(new Dominio(Dominio.TIPO_PARCEIRO, "Servidor", "SERVIDOR", 1L, Boolean.FALSE));
		list.add(new Dominio(Dominio.TIPO_PARCEIRO, "Orgão externo", "ORGAO_EXTERNO", 2L, Boolean.FALSE));
		list.add(new Dominio(Dominio.TIPO_PARCEIRO, "Portador", "PORTADOR", 3L, Boolean.FALSE));
		list.add(new Dominio(Dominio.TIPO_PARCEIRO, "Fornecedor", "FORNECEDOR", 4L, Boolean.FALSE));
		list.add(new Dominio(Dominio.TIPO_PARCEIRO, "Seguradora", "SEGURADORA", 5L, Boolean.FALSE));
		list.add(new Dominio(Dominio.TIPO_PARCEIRO, "Terceirizado", "TERCEIRIZADO", Dominio.TIPO_PARCEIRO_TERCEIRIZADO, Boolean.FALSE));
		list.add(new Dominio("tipoEndereco", "Residencial", "RESIDENCIAL", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoEndereco", "Comercial", "COMERCIAL", 2L, Boolean.FALSE));
		list.add(new Dominio("tipoPessoa", "Física", "FISICA", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoPessoa", "Jurídica", "JURIDICA", 2L, Boolean.FALSE));
		list.add(new Dominio("tipoTelefone", "Comercial", "COMERCIAL", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoTelefone", "Residencial", "RESIDENCIAL", 2L, Boolean.FALSE));
		list.add(new Dominio("tipoTelefone", "Celular", "CELULAR", 3L, Boolean.FALSE));
		list.add(new Dominio("tipoSexo", "Masculino", "MASCULINO", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoSexo", "Feminino", "FEMININO", 2L, Boolean.FALSE));
		list.add(new Dominio("tipoEstadoCivil", "Solteiro", "SOLTEIRO", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoEstadoCivil", "Casado", "CASADO", 2L, Boolean.FALSE));
		list.add(new Dominio("tipoEstruturaOrganizacional", "Unidade gestora", "UNIDADE_GESTORA", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoEstruturaOrganizacional", "Unidade administrativa", "UNIDADE_ADMINISTRATIVA", 2L, Boolean.FALSE));
		list.add(new Dominio("tipoEstruturaOrganizacional", "Unidade localizadora", "UNIDADE_LOCALIZADORA", 3L, Boolean.FALSE));
		list.add(new Dominio("tipoEstruturaOrganizacional", "Unidade requisitante", "UNIDADE_REQUISITANTE", 4L, Boolean.FALSE));
		list.add(new Dominio("tipoFeriado", "Data", "DATA", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoFeriado", "Período", "PERIODO", 2L, Boolean.FALSE));
		list.add(new Dominio("abrangenciaFeriado", "Mundial", "MUNDIAL", 1L, Boolean.FALSE));
		list.add(new Dominio("abrangenciaFeriado", "Nacional", "NACIONAL", 2L, Boolean.FALSE));
		list.add(new Dominio("abrangenciaFeriado", "Estadual", "ESTADUAL", 3L, Boolean.FALSE));
		list.add(new Dominio("abrangenciaFeriado", "Municipal", "MUNICIPAL", 4L, Boolean.FALSE));
		list.add(new Dominio("tipoExcecaoFeriado", "Folga", "FOLGA", 1L, Boolean.FALSE));
		list.add(new Dominio("tipoExcecaoFeriado", "Trabalho", "TRABALHO", 2L, Boolean.FALSE));
		
		this.dominioService.saveListIfNotExist(list);
		Logger.getLogger(StartupListenerTabelasCorp.class).info("Domínios executados - StartupListenerTabelasCorp!");
	}

	/**
	 * Método criado para criar todas as classes-parceiro
	 */
	private void criarClasseParceiro() {

		Dominio dominioParceiroColaborador = dominioService.findByChaveAndCodigo(Dominio.TIPO_PARCEIRO, Dominio.TIPO_PARCEIRO_COLABORADOR_SERVIDOR);
		Dominio dominioParceiroOrgaoExterno = dominioService.findByChaveAndCodigo(Dominio.TIPO_PARCEIRO, Dominio.TIPO_PARCEIRO_ORGAO_EXTERNO);
		Dominio dominioParceiroPortador = dominioService.findByChaveAndCodigo(Dominio.TIPO_PARCEIRO, Dominio.TIPO_PARCEIRO_PORTADOR);
		Dominio dominioParceiroFornecedor = dominioService.findByChaveAndCodigo(Dominio.TIPO_PARCEIRO, Dominio.TIPO_PARCEIRO_FORNECEDOR);
		Dominio dominioParceiroSeguradora = dominioService.findByChaveAndCodigo(Dominio.TIPO_PARCEIRO, Dominio.TIPO_PARCEIRO_SEGURADORA);
		Dominio dominioParceiroTerceiro = dominioService.findByChaveAndCodigo(Dominio.TIPO_PARCEIRO, Dominio.TIPO_PARCEIRO_TERCEIRIZADO);

		classeParceiroService.saveIfNotExist(new ClasseParceiro(dominioParceiroColaborador));
		classeParceiroService.saveIfNotExist(new ClasseParceiro(dominioParceiroOrgaoExterno));
		classeParceiroService.saveIfNotExist(new ClasseParceiro(dominioParceiroPortador));
		classeParceiroService.saveIfNotExist(new ClasseParceiro(dominioParceiroFornecedor));
		classeParceiroService.saveIfNotExist(new ClasseParceiro(dominioParceiroSeguradora));
		classeParceiroService.saveIfNotExist(new ClasseParceiro(dominioParceiroTerceiro));
		Logger.getLogger(StartupListenerTabelasCorp.class).info("ClasseParceiro executado - StartupListenerTabelasCorp!");
	}
	
	/**
	 *
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 */
	private void criarDefaultFile() {

		dominioJS = this.dominioService.findByChaveAndNome(TIPO_FILE, TIPO_FILE_JS);

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/EnderecoDirective.js", 132, dominioJS));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/EnderecoDirective.min.js", 132, dominioJS));

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/EnderecoUnicoDirective.js", 133, dominioJS));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/EnderecoUnicoDirective.min.js", 133, dominioJS));

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/ObservacaoDirective.js", 134, dominioJS));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/ObservacaoDirective.min.js", 134, dominioJS));

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/TelefoneDirective.js", 135, dominioJS));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/TelefoneDirective.min.js", 135, dominioJS));

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "repository/DocumentoRepository.js", dominioJS, 140));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "repository/DocumentoRepository.min.js", dominioJS, 141));

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "repository/AnexoRepository.js", dominioJS, 142));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "repository/AnexoRepository.min.js", dominioJS, 143));

		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/AssociarDocumentoDirective.js", dominioJS, 144));
		this.defaultFileService.saveIfNotExist(new DefaultFile(CIT_TABELAS_CORP_WEB_ANGULAR_CUSTOM + "directive/AssociarDocumentoDirective.min.js", dominioJS, 145));
		
		Logger.getLogger(StartupListenerTabelasCorp.class).info("DefaultFile executado - StartupListenerTabelasCorp!");
	}

	/**
	 *
	 * Método responsável por gerar internacionalização do módulo corporativo
	 *
	 * @author rogerio.cassimiro
	 *
	 */
	private void gerarInternacionalizacao() {

		dominioPT_BR = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		moduloSelecionado = this.moduloService.getModuloPorBaseUrl(BASE_URL_TABELAS_CORP, null);
		List<Internacionalizacao> internacionalizacaoList = new ArrayList<Internacionalizacao>();

		this.gerarLabel(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		this.gerarValidacao(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		this.gerarMensagem(dominioPT_BR, moduloSelecionado, internacionalizacaoList);

		internacionalizacaoService.saveListIfNotExist(internacionalizacaoList);
		
		Logger.getLogger(StartupListenerTabelasCorp.class).info("Internacionalização executada - StartupListenerTabelasCorp!");
	}

	private void gerarLabel(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.SELECIONE_MODULO", "Selecione o módulo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.SELECIONE_FUNCIONALIDADE", "Selecione a funcionalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.REVISAO", "Nº Revisão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DATA_OPERACAO", "Data Operação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.NOVO_REGISTRO", "Criou", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EDITOU_REGISTRO", "Alterou", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EXCLUI_REGISRO", "Excluiu", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TITULO_EXPORT_CSV_Audit", "CSV_AUDITORIA", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.HORA_OPERACAO", "Hora Operação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DETALHES_AUDITORIA", "Detalhes Auditoria", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EMPLAQUETADO", "Emplaquetado", dominio, modulo));
		/** Eric **/
		
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ADICIONAR_RESPONSAVEL", "Adicionar responsável", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CARGO", "Cargo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CODIGO_IBGE", "Código IBGE", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.COMPRAS_ELETRONICAS", "Compras eletrônicas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CONTATOS", "Contatos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_BAIRRO", "Dados de bairro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_CARACTERISTICA", "Dados de característica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_CARACTERISTICA", "Dados de cidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_ENDERECO", "Dados de endereço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_ESTADO", "Dados de estado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_LOCALIZACAO", "Dados de localização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_NIVEL_AUTORIDADE", "Dados do nível de autoridade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_PAIS", "Dados de país", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_PESSOA", "Dados da pessoa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_REGIAO", "Dados de região", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DEFINICOES_ALMOXARIFADO", "Definições almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DEFINICOES_ECM", "Definições ECM", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DEFINICOES_PATRIMONIO", "Definições patrimônio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.E_OBRIGATORIO", " é obrigatório(a)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EMITENTE", "Emitente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ENDERECOS", "Endereços", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ESTADO_CIVIL", "Estado civil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ESTRANGEIRO", "Estrangeiro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EXIBIR_APENAS_ATIVAS", "Exibir apenas estruturas ativas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EXPRESSAO_REGULAR", "Expressão regular", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.FILANTROPICO", "Filantrópico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.FILIAL", "Filial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.FUNCAO", "Função", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.HIERARQUIA", "Hierarquia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.IMPOSTO_SIMPLES", "Imposto simples", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.MAPA_EXCLUIDO_SALVO_SUCESSO", "Exclusão do Mapa Organizacional realizada com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.MAPA_ORGANIZACIONAL", "Mapa organizacional", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.PORTE", "Porte", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.RAMOS_ATIVIDADES", "Ramos de atividades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.SEGURADORA", "Seguradora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.SERVIDOR", "Servidor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.SEXO", "Sexo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TELEFONES", "Telefones", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TERCEIRIZADO", "Terceirizado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_COMISSAO", "Tipo comissão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_DADO", "Tipo dado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_DE_ESTRUTURA_ORGANIZACIONAL", "Tipo de estrutura organizacional", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_RESTRICAO", "Tipo restrição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ASSOCIAR_DOCUMENTOS", "Associar documentos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.BAIRRO", "Bairro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CEP", "CEP", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CIDADE", "Cidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CNPJ", "CNPJ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.COMPLEMENTO", "Complemento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CPF", "CPF", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ENDERECO_PRINCIPAL", "Endereço principal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.LOGRADOURO", "Logradouro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.MAPA_ORGANIZACIONAL_SALVO_SUCESSO", "Mapa Organizacional cadastrado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.PAIS", "País", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.REGIAO", "Região", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.REPRESENTANTE_LEGAL", "Representante legal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TELEFONE", "Telefone", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_ENDERECO", "Tipo de endereço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_PESSOA", "Tipo pessoa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.AUTOR", "Autor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CENTRO_CUSTO_PAI", "Centro de custo pai", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.CLASSIFICACAO", "Classificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DATA_NASCIMENTO", "Data de nascimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EMPRESA_TERCEIRIZADA", "Empresa terceirizada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ENDERECO", "Endereço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ESTRUTURA_SUPERIOR", "Estrutura superior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.INSCRICAO_ESTADUAL", "Inscrição estadual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.INSCRICAO_MUNICIPAL", "Inscrição municipal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.LOCALIZACAO", "Localização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.N_DOCUMENTO", "Nº do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.NOME_FANTASIA", "Nome fantasia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.NOME_MAE", "Nome da mãe", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.NOME_PAI", "Nome do pai", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.NUMERO_CONTRATO", "Número de contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.NUMERO_PASSAPORTE", "Número passaporte", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.OBSERVACAO", "Observação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.RAZAO_SOCIAL", "Razão social", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.RG", "RG", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.SITE", "Site", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_EMISSOR", "Tipo de emissor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TITULO_ELEITOR", "Título de eleitor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.EMISSOR", "Emissor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ESTADO", "Estado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_CENTRO_CUSTO", "Dados do centro de custo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.ATUALIZA_ESI", "Atualiza ESI", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.TIPO_NUMERICO", "Tipo numérico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.DADOS_UNIDADE_MEDIDA", "Dados da unidade de medida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.LABEL.QUANTIDADE_PADRAO", "Qtde. Padrão", dominio, modulo));
	}
	
	private void gerarValidacao(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.REGISTRO_BLOQUEADO_NAO_PODE_SER_ALTERADO", "Registro bloquado não pode ser alterado, tente desbloqueá-lo.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.CAMPO_DATA_INICIO_MAPA", "Só pode haver um mapa organizacional vigente, portanto, o novo mapa deve ter a data início no mínimo 2 dias superior a data início do último mapa vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.CPF_INVALIDO_CONTATO", "CPF do contato está inválido.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.CPF_OBRIGATORIO_REPRESENTANTE_LEGAL", "CPF é obrigatório para o representante legal.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.CPF_OBRIGATORIO_REPRESENTANTE_LEGAL", "Data de emissão não pode ser maior que a data atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.ERRO_EXCLUSAO_ESTRUTURA", "A Estrutura não pode ser removida pois contem vínculo com outra(s) entidade(s)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.PREENCHA_CAMPO_OBSERVACAO", "Preencha o campo observação!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.REPRESENTANTE_LEGAL_UNICO", "Só é permitido adicionar um representante legal para os contatos.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.SELECIONAR_UM_RESPONSAVEL", "Você precisa selecionar um responsável!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.ULTIMO_MAPA_VIGENTE", "O sistema deve sempre ter um Mapa Organizacional vigente, portanto esse Mapa não pode ser excluído. Caso precise de um novo Mapa é só inserir um mapa e o sistema imediatamente finalizará a vigência do mapa anterior.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.CENTRO_CUSTO_VINCULADO", "Centro de custo pai já está vinculado ao centro de custo selecionado para edição.", dominioPT_BR, moduloSelecionado));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.VALIDACAO.DATA_INICIO_MAPA_VIGENTE", "Data início não pode ser inferior a data início do Mapa Organizacional vigente!", dominioPT_BR, moduloSelecionado));
		
	}
	
	private void gerarMensagem(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.UNIDADES_COM_RESPONSAVEL_PODE_SER_LOCALIZADORA", "Apenas unidades com responsável definido podem ser localizadoras de bens!", dominioPT_BR, moduloSelecionado));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_DATA_FIM", "Ao alterar data fim de uma estrutura, o processamento poderá levar um tempo até preencher a data fim das estruturas filhas, se houver. Deseja prosseguir?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_BAIRRO", "Confirma a exclusão do bairro?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_CARACTERISTICA", "Confirma a exclusão da característica?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_CIDADE", "Confirma a exclusão da cidade?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_ENDERECO", "Confirma a exclusão do endereço?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_ESTADO", "Confirma a exclusão do estado?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_LOCALIZACAO", "Confirma a exclusão da localização?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_LOCALIZACAO", "Confirma a exclusão do país?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.CONFIRMA_EXCLUIR_REGIAO", "Confirma a exclusão da região?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.EXCLUSAO_MAPA_ORGANIZACIONAL", "O mapa organizacional tem a função de vincular as operações do sistema a um determinado conceito/período. Ao confirmar a operação o sistema irá excluir o mapa vigente e ativar imediatamente o anterior a esse. Deseja realmente excluir o mapa organizacional?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.NENHUMA_ESTRUTURA_CADASTRADA", "Nenhuma unidade cadastrada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SEM_OBSERVACOES", "Não possui observações!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_BAIRRO", "Bairro salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_CARACTERISTICA", "Característica salva com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_CARACTERISTICA_EXCLUIDO", "Característica excluída com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_CIDADE", "Cidade salva com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_CIDADE_EXCLUIDO", "Cidade excluída com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_ENDERECO", "Endereço salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_ESTADO", "Estado salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_ESTADO_EXCLUIDO", "Estado excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_ESTRUTURA_ALMOXARIFADO", "Definições de Almoxarifado salvas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_ESTRUTURA_ECM", "Definições de ECM salvas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_ESTRUTURA_PATRIMONIO", "Definições de Patrimônio salvas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_LOCALIZACAO", "Localização salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_LOCALIZACAO_EXCLUIDO", "Localização excluída com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_PAIS", "País salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_PAIS_EXCLUIDO", "País excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_REGIAO", "Região salva com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.SUCESSO_REGIAO_EXCLUIDO", "Região excluída com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.HIERARQUIA_UNICA", "Não pode existir mais de um nível de autoridade com a mesma hierarquia!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CORPORATIVO.MSG.QTDE_VALOR_PADRAO_NAO_PERMITIDO", "Quantidade padrão não permitida para esta operação!", dominio, modulo));

	}

}
