package br.com.centralit.controller.impl;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabase;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseBuilder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.controller.GenericControllerTest;
import br.com.centralit.controller.test.utils.TestUtil;
import br.com.centralit.framework.model.SearchParams;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.model.arquitetura.PersistentObject;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 18/06/2015 - 16:54:38
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.ribeiro@centralit.com.br">lucas.ribeiro@centralit.com.br</a>)
 *	
 * @param <T>
 */
public abstract class GenericControllerTestImpl<T extends PersistentObject> implements GenericControllerTest {

	public static final Logger LOG = Logger.getLogger(GenericControllerTestImpl.class);

	protected abstract SearchParams getSearchParams();

	protected abstract String getRequestMapping();

	protected abstract void initMocks();

	protected static MockMvc mockMvc;

	protected static EmbeddedDatabase database;

	@Autowired
	private WebApplicationContext webApplicationContext;

	@Autowired
	protected UsuarioService usuarioService;

	/**
	 * Inicializa a base de dados em mem&oacute;ria e templates de objetos.
	 * */
	@BeforeClass
	public static void setUp(){
		database = new EmbeddedDatabaseBuilder().addDefaultScripts().build();
	}

	/**
	 * Inicializa mocks do SUT e inicializa contexto de spring-mvc controllers mockados.
	 * */
	@Before
	public void init() {
		initMocks();
		SecurityContextMock();
		mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).build();
	}

	/**
	 * Ao final de todos os testes desliga a base de dados em memoria.
	 * */
	@AfterClass
	public static void tearDown() throws Exception {
		database.shutdown();
	}

	/**
	 * Cria um contexto fake do Spring para testes unit&aacute;rios
	 *
	 */
	public void SecurityContextMock() {

		Usuario usuario = usuarioService.find(1L);
		Authentication authentication = Mockito.mock(Authentication.class);
		SecurityContext securityContext = Mockito.mock(SecurityContext.class);

		LOG.info("Carregando o seguinte usuario no contexto de segurança: /n"
				+ usuario.getUsername() + "/n"
				+ usuario.getOrganizacao());

		Mockito.when(securityContext.getAuthentication()).thenReturn(authentication);
		Mockito.when(authentication.getPrincipal()).thenReturn(usuario);
		SecurityContextHolder.setContext(securityContext);
		SecurityContextHolder.getContext().getAuthentication().getPrincipal();
	}

	@SuppressWarnings("unchecked")
	@Test
	@Override
	public void findGridTest() throws Exception {

		LOG.info("==================== Inicializando teste findGrid, que retorna objetos atraves de paramentro ==================== ");

		LOG.info("SUT: " + getNameEntity());

		T entity = (T) getSearchParams();

		LOG.info(TestUtil.convertObjectToJsonString(entity));

		mockMvc.perform(
				post(getRequestMapping() + "/getPage")
				.contentType(TestUtil.APPLICATION_JSON_UTF8)
				.content(TestUtil.convertObjectToJsonString(entity))
		)
				.andExpect(status().isOk())
				.andDo(print());

		LOG.info("==================== Finalizando teste findGrid, que retorna objetos atraves de paramentro ==================== ");
	}

	@Test
	@Override
	public void getListTest() throws Exception {

		LOG.info("==================== Iniciando teste getList, que retorna todos os objetos ==================== ");

		LOG.info("SUT: " + getNameEntity());

		mockMvc.perform(get(getRequestMapping()))
				.andExpect(status().isOk())
				.andDo(print())
				.andExpect(content().contentType(MediaType.APPLICATION_JSON));

		LOG.info("==================== Finalizando teste getList, que retorna todos os objetos ==================== ");

	}

	@Test
	@Override
	public void saveTest() throws Exception {

		LOG.info("==================== Inicializando teste de salvamento ====================");
		LOG.info("SUT: " + getNameEntity());

		String json = TestUtil.getJsonFromFile(getNameEntity());
		mockMvc.perform(
				post(getRequestMapping())
				.content(json)
				.contentType(MediaType.APPLICATION_JSON)
		)
				.andExpect(status().isOk())
				.andDo(print())
				.andExpect(content().contentType(TestUtil.APPLICATION_JSON_UTF8));

		LOG.info("==================== Finalizando teste de salvamento ==================== ");
	}

	@Override
	public void updateTest() throws Exception {}

	/**
	 * Testa chamada de uma requisi&ccedil;&atilde;o DELETE para um objeto pr&eacute;-existente na base de dados em mem&oacute;ria
	 * */
	@Test
	@Override
	public void deveDeletarEntidadeTest() throws Exception {

		LOG.info("==================== Inicializando teste de remocao de entidades ==================== ");
		LOG.info("SUT: " + getNameEntity());

		mockMvc.perform(
				delete(getRequestMapping() + "/{id}", 1L))
				.andExpect(status().isOk())
				.andExpect(content().contentType(TestUtil.APPLICATION_JSON_UTF8))
				.andDo(print());
		LOG.info("==================== Finalizando teste de remocao de entidades ==================== ");
	}

//	@Test
	@Override
	public void removeAllTest() throws Exception {
		LOG.info("==================== Inicializando teste removeAll, que faz remocao logica por ids ==================== ");
		LOG.info("SUT: " + getNameEntity());

		List<Long> listIds = new ArrayList<Long>(Arrays.asList(8L,9L,10L));

		mockMvc.perform(
				post(getRequestMapping() + "/removeAll")
				.contentType(TestUtil.APPLICATION_JSON_UTF8)
				.content(TestUtil.convertObjectToJsonString(listIds))
		)
				.andExpect(status().isOk())
				.andDo(print());

		LOG.info("==================== Finalizando teste removeAll, que faz remocao logica por ids ==================== ");
	}

//	@Test
	@Override
	public void findAutoCompleteTest() throws Exception {}

	@SuppressWarnings("unchecked")
	protected String getNameEntity() {
		final Type type[] = ((ParameterizedType) this.getClass().getGenericSuperclass()).getActualTypeArguments();
		Class<T> classs = (Class<T>) type[0];
		return classs.getSimpleName();
	}
}
